module gamesdb;
private import pspgamelist;
private import etc.c.sqlite3;
import std.exception;
import std.conv : to;
import std.string : fromStringz, toStringz, empty;
import std.format;

class SQLiteException : Exception {
	this(int code, string msg1, string msg2, string file = __FILE__, size_t line = __LINE__) {
		super("SQLite3 error " ~ to!string(code) ~ (msg1.empty ? "" : " ") ~ msg1 ~ ": " ~ msg, file, line);
	}
}

static this() {
	sqlite3* db;
	scope(exit) { if (db) { sqlite3_close(db); } }

	const int open_ret = sqlite3_open(":memory:", &db);
	// Open an in-memory database to use as a handle for loading the memvfs extension
	if (open_ret != SQLITE_OK) {
		throw new SQLiteException(open_ret, "opening in-memory database", sqlite3_errmsg(db).fromStringz.idup);
	}

	sqlite3_enable_load_extension(db, 1);
	char* err;
	const int load_ret = sqlite3_load_extension(db, "./libmemvfs.so", null, &err);
	if (load_ret != SQLITE_OK) {
		throw new SQLiteException(load_ret, "loading memvfs extension", err.fromStringz.idup);
	}
}

class GamesDB {
	this() {

		void* deleme = cast(void*)g_sqlite_db;
		string memdb = "file:gamesdb?ptr=0x%s&sz=%d&freeonclose=false".format(deleme, g_sqlite_db.length);
		const int open_ret = sqlite3_open_v2(memdb.toStringz(), &m_db, SQLITE_OPEN_READWRITE | SQLITE_OPEN_URI, "memvfs".toStringz());
		if (open_ret)
			throw new SQLiteException(open_ret, "opening in-memory database", sqlite3_errmsg(m_db).fromStringz.idup);

	}

	string find_by_serial (string serial) {
		string sql = "SELECT Title FROM GameList WHERE Serial=?;";
		sqlite3_stmt* statement;
		scope(exit) {sqlite3_finalize(statement);}
		const int prepare_ret = sqlite3_prepare_v2(m_db, sql.ptr, cast(int)sql.length, &statement, null);

		if (prepare_ret)
			throw new SQLiteException(prepare_ret, "", sqlite3_errstr(prepare_ret).fromStringz.idup);

		sqlite3_bind_text(statement, 1, serial.ptr, cast(int)serial.length, SQLITE_STATIC);

		bool done = false;
		while (!done) {
			int bytes;
			const(char)* text_ptr;
			string text;
			const int step_ret = sqlite3_step(statement);
			switch(step_ret) {
			case SQLITE_ROW:
				bytes = sqlite3_column_bytes(statement, 0);
				text_ptr = sqlite3_column_text(statement, 0);
				text = text_ptr[0..cast(size_t)bytes].idup;
				return text;
			case SQLITE_DONE:
				done = true;
				break;
			default:
				throw new SQLiteException(step_ret, "", sqlite3_errstr(step_ret).fromStringz.idup);
			}
		}

		return "";
	}

	~this() {
		if (m_db) {
			sqlite3_close(m_db);
		}
	}

private:
	sqlite3* m_db = null;
}
