/* @(#)scsi.c	1.35 12/12/02 Copyright 1997-2012 J. Schilling */
#include <schily/mconfig.h>
#ifndef lint
static	UConst char sccsid[] =
	"@(#)scsi.c	1.35 12/12/02 Copyright 1997-2012 J. Schilling";
#endif
/*
 *	Copyright (c) 1997-2012 J. Schilling
 */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; see the file COPYING.  If not, write to the Free Software
 * Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef	USE_SCG

#include <schily/stdio.h>
#include <schily/standard.h>
#include <schily/stdlib.h>
#include <schily/unistd.h>
#include <schily/schily.h>
#include <schily/nlsdefs.h>
#include <schily/string.h>

//#include "mkisofs.h"
#define SECTOR_SIZE (2048)
BOOL ignerr;

#include <scg/scsireg.h>
#include <scg/scsitransp.h>

#include "scgcmd/libscgcmd.h"
#include "cdrdeflt/cdrdeflt.h"

/*
 * NOTICE:	You should not make BUF_SIZE more than
 *		the buffer size of the CD-Recorder.
 *
 * Do not set BUF_SIZE to be more than 126 KBytes
 * if you are running cdrecord on a sun4c machine.
 *
 * WARNING:	Philips CDD 521 dies if BUF_SIZE is to big.
 */
#define	BUF_SIZE	(62*1024)	/* Must be a multiple of 2048	   */

enum ScsiOpenErr {
	ScsiOpenErr_Ok = 0,
	ScsiOpenErr_CantOpen, /*Cannot open SCSI driver*/
	ScsiOpenErr_CantIO, /*Cannot get SCSI I/O buffer*/
	ScsiOpenErr_Timeout,
	ScsiOpenErr_Invalid
};

struct scsi_dev {
	FILE *file;
	SCSI *scgp;
	int err;
};

LOCAL	long	bufsize;		/* The size of the transfer buffer */

EXPORT int	readsecs	__PR((struct scsi_dev* src, UInt32_t startsecno, void *buffer, int sectorcount));
EXPORT BOOL scsidev_has_error __PR((struct scsi_dev *dev));
EXPORT int scsidev_error_code __PR((struct scsi_dev *dev));
EXPORT const char* scsidev_error_desc __PR((int code));

#ifdef	USE_SCG
EXPORT struct scsi_dev* scsidev_open __PR((char *path));
EXPORT int scsidev_close __PR((struct scsi_dev *f));
#endif
EXPORT struct scsi_dev* scsidev_file_open __PR((const char *path));

EXPORT int
readsecs(src, startsecno, buffer, sectorcount)
	struct scsi_dev *src;
	UInt32_t startsecno;
	void	*buffer;
	int	sectorcount;
{
	int	f;
	int	secsize;	/* The drive's SCSI sector size		*/
	long	amount;		/* The number of bytes to be transfered	*/
	long	secno;		/* The sector number to read from	*/
	long	secnum;		/* The number of sectors to read	*/
	char	*bp;
	long	amt;

	if (src->file == NULL) {
		/*
		 * We are using the standard CD-ROM sectorsize of 2048 bytes
		 * while the drive may be switched to 512 bytes per sector.
		 *
		 * XXX We assume that secsize is no more than SECTOR_SIZE
		 * XXX and that SECTOR_SIZE / secsize is not a fraction.
		 */
		secsize = src->scgp->cap->c_bsize;
		amount = sectorcount * SECTOR_SIZE;
		secno = startsecno * (SECTOR_SIZE / secsize);
		bp = buffer;

		while (amount > 0) {
			amt = amount;
			if (amount > bufsize)
				amt = bufsize;
			secnum = amt / secsize;

			if (read_scsi(src->scgp, bp, secno, secnum) < 0 ||
						scg_getresid(src->scgp) != 0) {
#ifdef	OLD
				return (-1);
#else
				comerr(_("Read error on old image\n"));
#endif
			}

			amount	-= secnum * secsize;
			bp	+= secnum * secsize;
			secno	+= secnum;
		}
		return (SECTOR_SIZE * sectorcount);
	}

	f = fileno(src->file);

	if (lseek(f, (off_t)startsecno * SECTOR_SIZE, SEEK_SET) == (off_t)-1) {
		comerr(_("Seek error on old image\n"));
	}
	if ((amt = read(f, buffer, (sectorcount * SECTOR_SIZE)))
			!= (sectorcount * SECTOR_SIZE)) {
		if (ignerr) {
			if (amt < 0)
				amt = 0;
			fillbytes(&((char *)buffer)[amt],
					(sectorcount * SECTOR_SIZE) - amt, '\0');
			return (sectorcount * SECTOR_SIZE);	/* Should we cheat here too? */
		}
		if (amt < 0)
			comerr(_("Read error on old image\n"));
		comerrno(EX_BAD, _("Short read on old image\n")); /* < secnt aber > 0 */
	}
	return (sectorcount * SECTOR_SIZE);
}

EXPORT BOOL scsidev_has_error (src)
	struct scsi_dev *src;
{
	if (NULL == src || ScsiOpenErr_Ok != src->err)
		return TRUE;
	else
		return FALSE;
}

EXPORT int scsidev_error_code (src)
	struct scsi_dev *src;
{
	if (NULL == src)
		return ScsiOpenErr_Invalid;
	else
		return src->err;
}

EXPORT const char* scsidev_error_desc (code)
	int code;
{
	switch (code) {
	case ScsiOpenErr_Timeout:
		return "Timeout waiting for device";
	case ScsiOpenErr_CantIO:
		return "Cannot get SCSI I/O buffer";
	case ScsiOpenErr_CantOpen:
		return "Cannot open SCSI driver";
	case ScsiOpenErr_Invalid:
		return "Invalid device handle";
	default:
		return "";
	}
}

EXPORT struct scsi_dev*
scsidev_file_open (path)
	const char* path;
{
	struct scsi_dev *retval;

	retval = (struct scsi_dev*)malloc(sizeof(*retval));
	memset(retval, 0, sizeof(*retval));
	retval->file = fopen(path, "rb");
	return retval;
}

EXPORT struct scsi_dev*
scsidev_open(path)
	char	*path;
{
	char	errstr[80];
	char	*buf;	/* ignored, bit OS/2 ASPI layer needs memory which */
			/* has been allocated by scsi_getbuf()		   */
	struct scsi_dev *retval;

	/*
	 * Call scg_remote() to force loading the remote SCSI transport library
	 * code that is located in librscg instead of the dummy remote routines
	 * that are located inside libscg.
	 */
	scg_remote();

	cdr_defaults(&path, NULL, NULL, NULL, NULL);
			/* path, debug, verboseopen */
	retval = (struct scsi_dev*)malloc(sizeof(*retval));
	memset(retval, 0, sizeof(*retval));

	retval->scgp = scg_open(path, errstr, sizeof (errstr), 0, 0);
	if (retval->scgp == 0) {
		retval->err = ScsiOpenErr_CantOpen;
		return retval;
	}

	bufsize = scg_bufsize(retval->scgp, BUF_SIZE);
	if ((buf = scg_getbuf(retval->scgp, bufsize)) == NULL) {
		retval->err = ScsiOpenErr_CantIO;
		scg_close(retval->scgp);
		retval->scgp = NULL;
		return retval;
	}

	bufsize = (bufsize / SECTOR_SIZE) * SECTOR_SIZE;

	allow_atapi(retval->scgp, TRUE);

	if (!wait_unit_ready(retval->scgp, 60)) { /* Eat Unit att / Wait for drive */
		retval->err = ScsiOpenErr_Timeout;
		return retval;
	}

	retval->scgp->silent++;
	read_capacity(retval->scgp);	/* Set Capacity/Sectorsize for I/O */
	retval->scgp->silent--;

	return retval;
}

EXPORT int
scsidev_close(struct scsi_dev *f)
{
	int retval;

	if (NULL == f)
		return 0;

	retval = 0;
	if (f->scgp != NULL) {
		retval = scg_close(f->scgp);
	}
	if (f->file != NULL) {
		retval = fclose(f->file);
	}
	free(f);
	return (retval);
}

#endif	/* USE_SCG */
